package ExRand;
/**
 *
 * @author TeleVox
 */
import java.io.*;
import com.televox.TeleToken;

public class ExRand {
    public static void main(String[] args) {

        TeleToken TVTT;
        String RandInitFileName = "MyRandInitFile.bin";
        byte[][] RandInitTblB = new byte[4][16];
        int[][] RandInitTbl = new int[4][16];
        int m, n, TTime;        
        int[] RandTbl = new int[16];        

        TVTT = new TeleToken();

        // przygotowanie tablicy inicjującej wypełnionej możliwie nieprzewidywalną wartościami / preparing of init table, filling with possibly random values
        //  - metoda przykładowa prosta ale nie zalecana, należy zastosować własną metodę / example method is too simple, own method urged

        n = 0;
        while (n<TeleToken.TVTT_RAND_INIT_TBL_SIZE) {
            RandInitTbl[n/16][n%16] = (int) System.currentTimeMillis() & 0xFF;
            n++;
        }
        
        File file1 = new File(RandInitFileName);
        // odczytanie pliku inicjującego (o ile istnieje) / reading of init file (if it exists)
        try {            
            if (file1.exists()){
                FileInputStream fin = new FileInputStream(file1);
                for (n=0;n<4;n++) {
                    fin.read(RandInitTblB[n], 0, TeleToken.TVTT_RAND_INIT_TBL_SIZE / 4);
                    for (m=0; m<16; m++) RandInitTbl[n][m] = RandInitTblB[n][m] & 0xFF;
                }
                // UWAGA! Nie następuje potwierdzenie poprawności odczytu ponieważ jeśli odczyt się nie uda to tablica
                //  jest już wypełniona wartościami możliwie nieprzewidywalnymi
                //  - jest to metoda prosta ale nie zalecana, w rozwiązaniu zalecanym powinno być zaimplementowane:
                //      * sprawdzenie poprawności odczytu
                //      * szyfrowanie pliku z tablicą inicjalizującą
                //      * sprawdzenie oryginalności pliku (suma kontrolna, podpis cyfrowy, etc.)

                // Important! There is no confirmation of successful read operation, because if it failed the table would be already filled with random values
                // This method is simple but not recommended. In real world solution there should be also:
                // * check of read status
                // * encryption of file with init table
                // * check of file's genuineness (checksum, digital sign, etc.)
                fin.close();

            }                        
        } catch(IOException e){};

        // zainicjowanie Generatora liczb pseudolosowych / initialize random number generator
        TVTT.RandInit(RandInitTbl);

        // zapisanie nowej tablicy inicjującej generator / saving of new initialization table
        TVTT.RandGetInit(RandInitTbl);
        try {
            FileOutputStream out = new FileOutputStream(file1);
            for (n=0;n<4;n++) {
                for (m=0; m<16; m++) RandInitTblB[n][m] = (byte) RandInitTbl[n][m];
                out.write(RandInitTblB[n], 0, TeleToken.TVTT_RAND_INIT_TBL_SIZE / 4);
            }

            // UWAGA! nie następuje przeprowadzenie poprawności zapisu
            //  - należy zastosować własną obsługę błędu zapisu zależną od potrzeb

            // Important! There is no confirmation of successful read operation
            out.close();
        } catch(IOException e){};

        // powyzsze kroki konieczne żeby program przy następnym uruchomieniu
        // wystartował z innym stanem generatora liczb losowych
        //  - wykonywane na wypadek zakończenia programu bez zapisu ostatniego stanu generatora
        
        // above steps are necessary for the application to start the pseudo random
        // generator with different seed next time
        // in case of exiting application without saving state


        // zbieranie wartości losowych
        // ta operacja powinna być wykonywana w głównej pętli obsługi wiadomości
        // zdarzeniami moga być np.:
        //  * moment wciśnięcia klawisza na klawiaturze (np. odczytany przez System.currentTimeMillis())
        //  * kod wciśniętego klawisza
        //  * moment wciśnięcia przycisku myszki (np. odczytany przez System.currentTimeMillis())
        //  * moment poruszenia myszki (np. odczytany przez System.currentTimeMillis())
        //  * kierunek przesunięcia myszki
        //  * inne mniej lub bardziej nieprzewidywalne zdażenia wystepujące w systemie
        // poniższa procedura może być przydatna przy pierwszym uruchomieniu programu gdy
        // nie ma jeszcze tablicy inicjującej generator liczb pseudolosowych

        // collecting of random events
        // this operation should be executed in main application event loop
        // these messages could be for example:
        // * time of press key event (returned by GetTickCount())
        // * key code of pressed key
        // * time of press mouse button event (returned by GetTickCount())
        // * time of mouse move event (returned by GetTickCount())
        // * direction of mouse move
        // * different system events
        // following procedure could be useful at first application start, when
        // there is no pseudo random generator initialization table yet

        System.out.println("Generowanie liczb pseudolosowych:");        
        
        TTime = (int) (System.currentTimeMillis()) + 10000;
        n = 0;
        while ((int) System.currentTimeMillis()<TTime) {
            n++;            
                
            TVTT.Randomize(0, (Math.abs((int) System.currentTimeMillis()) & 0xFF));    // jako pierwsze źródło przyjęto czas (ms) / time (ms) as first random source
            TVTT.Randomize(1, (Math.abs((int) System.nanoTime()) & 0xFF));             // jako drugie źródło przyjęto czas (ns) / time (ns) as second random source
            TVTT.Randomize(2, (int) n);                                                // jako trzecie źródło przyjęto obrót pętli / loop iteration as third random source
            
            try {      
                   Thread.sleep(RandInitTbl[Math.abs((int)System.currentTimeMillis() % 4)]
                                           [Math.abs((int)System.nanoTime() % 16)]);
                 } catch (InterruptedException t){}

            TVTT.Rand(RandTbl, 0);                                              //wygenerowanie i wyswietlenie liczb pseudolosowych / generation and display of pseudo random data
            System.out.print("\t");
            for (m=0; m<16; m++) System.out.print(String.format("%02X ", RandTbl[m]));
            System.out.println();

        }        

        // zapisanie stanu generatora liczb pseudolosowych / saving state of pseudo random generator
        TVTT.RandGetInit(RandInitTbl);
        try {
            FileOutputStream out = new FileOutputStream(file1);
            for (n=0;n<4;n++) {
                for (m=0; m<16; m++) RandInitTblB[n][m] = (byte) RandInitTbl[n][m];
                out.write(RandInitTblB[n], 0, TeleToken.TVTT_RAND_INIT_TBL_SIZE / 4);
            }

            // UWAGA! nie następuje przeprowadzenie poprawności zapisu
            //  - należy zastosować własną obsługę błędu zapisu zależną od potrzeb

            // Important! There is no confirmation of successful read operation
            out.close();
        } catch(IOException e){};
    }
}